#define _XTAL_FREQ 4000000
#include <htc.h>

__CONFIG(FOSC_XT & MCLRE_OFF & BOREN_OFF & PWRTE_ON & WDTE_OFF & LVP_OFF);

// connect LED cathode to ground
#define LED_ON() PORTAbits.RA4 = 0
// connect LED cathode to Vdd
#define LED_OFF() PORTAbits.RA4 = 1

volatile union {
  struct {
    unsigned timer1pass :1; // define one bit
    unsigned reserved   :7;
  };
} flags;
volatile unsigned char sec10;
volatile unsigned int sec;

void interrupt tc_int(void){
  if (PIR1bits.TMR1IF ) { // did timer 1 interrupt occur?
    PIR1bits.TMR1IF = 0;
    // add 0xb1 to TMR1L
	#asm
      movlw		0xb1
      addwf		_TMR1L
	#endasm
    TMR1H = TMR1H + 0x3C;
    if(flags.timer1pass){ // second time
      sec10++;
      if (sec10>9){
        sec10 = 0;
        if(sec > 0){
          sec--;
        }
      }
    }
    if (sec>0){
      // blink while counting
      if(sec10>=5){
        LED_OFF();
      }else{
        LED_ON();
      }
    }else{
      // off while not counting
      LED_OFF();
    }
    PORTAbits.RA2 = flags.timer1pass;
    flags.timer1pass = ~flags.timer1pass; // toggle the flag
  }

  if (INTCONbits.INTF) { // did external interrupt occur?
    INTCONbits.INTF = 0;
    sec10 = 0;
    sec = 5; // 60 seconds = 1 minute. you can set this up to 65535
              // seconds which is more than 18 hours, or even make it
              //  long int which could be set to more than 136 years.
  }

  // process other interrupt sources here, if required
}

void init(void){

  PCONbits.OSCF = 0; // set internal oscillator to 48kHz

  // by default all ports are set to inputs and analog functions are enabled

  CMCON = 0x07; // thurn the comparator off to enable digital IO

  TRISAbits.TRISA2 = 0; // set RA3 as output - debug signal 10Hz
  TRISAbits.TRISA3 = 0; // set RA3 as output
  TRISAbits.TRISA4 = 0; // set RA4 as output
  TRISBbits.TRISB0 = 1; // set RB0 as input
  RA3 = 0; // we start with the relay off
  RA4 = 0; // we start with the LED off

  // external interrupt configuration
  OPTION_REGbits.nRBPU = 0; // enable port b pull-ups
  OPTION_REGbits.INTEDG = 0; // interrupt on falling edge of RB0
  INTCONbits.INTE = 1; // enable external interrupt

  // timer 1 setup
  T1CONbits.T1OSCEN = 0; // we won't use the oscillator we'll use:
  T1CONbits.TMR1CS = 0; // T1 will use internal instruction cycle clock (Fosc/4)
  T1CONbits.T1CKPS = 0; // pre-scale 1:2
  TMR1H = 0x3C;
  TMR1L = 0xAF; // 15536 = 3CB0 in hex, 3C is the high byte, B0 is the low.

  INTCONbits.PEIE = 1; // enable peripheral interrupts (for timer1 int)
  PIE1bits.TMR1IE = 1; // enalbe timer 1 interrupt

  PIR1bits.TMR1IF = 0; // clear int flags before enabling interrupts
  INTCONbits.INTF = 0;
  INTCONbits.GIE = 1; //enable interrupts

  // short blink to visualize that the timer has turned on
  LED_ON();
  __delay_ms(100);  
  LED_OFF();

  flags.timer1pass = 0;
  sec = 0;
  sec10 = 0;

  // ready ?
  T1CONbits.TMR1ON = 1; // Go ... starts the timer.
}


void main(void){
  init();
  while(1){ // repeat forever
    if(sec == 0){
      PORTAbits.RA3 = 0; // turn the relay off
      LED_OFF(); // turn the LED off
    }else{
      PORTAbits.RA3 = 1; // turn the relay on
    }
  } //while
}
